import collections
import cv2
import math
import numpy as np


class Shred(collections.namedtuple('_Shred',
            'contour features name piece_fname features_fname '
            'piece_in_context_fname simplified_contour sheet img_roi '
            'tags_suggestions')):
    """Stores the shred information, incl. tags suggestions."""
    def __repr__(self):
        return '<Shred #%s of sheet %s>' % (self.name, self.sheet)


class Sheet(object):
    """Represents a single sheet with shred extraction methods.

    Public API includes initializer and get_shreds() method.
    """

    _backgrounds = [
        # DARPA SHRED task #1
        [
            [np.array([30, 190, 180]), np.array([180, 255, 255])],
            [np.array([0, 240, 170]), np.array([50, 255, 255])]
        ],
        # Pink
        [[np.array([160, 50, 210]), np.array([200, 150, 255])]],

        # Denys $100 paper background
        [
            [np.array([0, 100, 200]), np.array([20, 255, 255])],
            [np.array([140, 120, 200]), np.array([250, 255, 255])],
        ],

        # AB's brown paper
        [
            [np.array([0, 70, 120]), np.array([30, 140, 200])],
        ]
    ]

    def __init__(self, orig_image, dpi, save_image):
        """Initializes a Sheet instance.

        Args:
            orig_image: cv.Mat instance with the original sheet image.
            dpi: optional (x resolution, y resolution) tuple or None.
                If set to None, will try to guess dpi.
            save_image: A callback to save debug images with args (name, img)
        """

        self._shreds = None
        self.orig_img = orig_image

        self.save_image = save_image

        self._fg_mask = None
        self._shreds = None

        if dpi is None:
            self.res_x, self.res_y = self._guess_dpi()
        else:
            self.res_x, self.res_y = dpi

    def px_to_mm(self, px):
        """Convert given value in px to a value in millimetres according to
        sheet's DPI

        Args:
            px: integer, value in pixels

        Returns:
            value in millimetres
        """
        return float(px) / self.res_x * 25.4

    def get_shreds(self, feature_extractors, sheet_name):
        """Detects shreds in the current sheet and constructs Shred instances.

        Caches the results for further invocations.

        Args:
            feature_extractors: iterable of AbstractShredFeature instances to
                use for shreds feature assignment.
            sheet_name: string, included in shred attributes.

        Returns:
            list of Shred instances.
        """
        if self._shreds is None:
            shreds = []
            contours, _ = cv2.findContours(self._foreground_mask,
                                           cv2.RETR_EXTERNAL,
                                           cv2.CHAIN_APPROX_SIMPLE)
            for i, contour in enumerate(contours):
                shred = self._make_shred(contour, i, feature_extractors,
                                         sheet_name)
                if shred is not None:
                    shreds.append(shred)
            self._shreds = shreds
        return self._shreds

    def _guess_dpi(self):
        # Note: this is inconsistent for images with larger y dimension:
        # if they have exif, dpi is returned for (x, y), otherwise it's (y, x).

        # Let's assume that we are dealing with A4 (8.27 in x 11.7) and try to
        # guess it
        w, h = min(self.orig_img.shape[:2]), max(self.orig_img.shape[:2])
        xres, yres = w / 8.27, h / 11.7

        # Will suffice for now
        if max(xres, yres) / min(xres, yres) > 1.1:
            raise ValueError("Dpi is not provided and can't be guessed.")

        return int(round(xres, -2)), int(round(yres, -2))

    def _detect_scanner_background(self, img):
        # Method returns a mask describing detected scanner background
        # (gray borders around colored sheet where shreds are glued).
        # Problem here is that colored sheet might have borders of different
        # sizes on different sides of it and we don't know the color of the
        # sheet.
        # Also sheets don't always have perfectly straight edges or are
        # slightly rotated against edges of the scanner.
        # Idea is relatively simple:

        # Convert image to LAB and grab A and B channels.
        fimg = cv2.cvtColor(img, cv2.cv.CV_BGR2Lab)
        _, a_channel, b_channel = cv2.split(fimg)

        def try_method(fimg, border, aggressive=True):
            fimg = cv2.copyMakeBorder(fimg, 5, 5, 5, 5, cv2.BORDER_CONSTANT,
                                      value=border)

            # Flood fill supposed background with white (255).
            if aggressive:
                cv2.floodFill(fimg, None, (10, 10), 255, 1, 1)
            else:
                cv2.floodFill(fimg, None, (10, 10), 255, 2, 2,
                              cv2.FLOODFILL_FIXED_RANGE)

            # Binarize image into white background and black everything else.
            _, fimg = cv2.threshold(fimg, 254, 255, cv2.THRESH_BINARY)

            hist = cv2.calcHist([fimg], [0], None, [2], [0, 256])
            return fimg, hist

        options = [
            [a_channel, 127],
            [b_channel, 134],
            [a_channel, 127, False],
            [b_channel, 134, False],
        ]

        # And then try to add a border of predefined color around each channel
        # and flood fill scanner background starting from most
        # aggressive flood fill settings to least aggressive.
        for i, opt in enumerate(options):
            fimg, hist = try_method(*opt)
            # First setting that doesn't flood that doesn't hurt colored sheet
            # too badly wins.
            bg_ratio = hist[1] / sum(hist)
            if bg_ratio < 0.2:
                break

        # Then we dilate it a bit.
        fimg = cv2.morphologyEx(fimg, cv2.MORPH_DILATE,
                                np.ones((5, 5), np.uint8), iterations=2)
        fimg = fimg[5:-5, 5:-5]

        # Searching for a biggest outter contour
        contours, _ = cv2.findContours(cv2.bitwise_not(fimg),
                                       cv2.RETR_EXTERNAL,
                                       cv2.CHAIN_APPROX_SIMPLE)
        main_contour = np.array(map(cv2.contourArea, contours)).argmax()

        # build a convex hull for it
        hull = cv2.convexHull(contours[main_contour])

        # And make a mask out of it.
        fimg = np.zeros(fimg.shape[:2], np.uint8)
        cv2.fillPoly(fimg, [hull], 255)
        fimg = cv2.bitwise_not(fimg)

        # Cannot say I'm satisfied with algo but using grabcut here seems too
        # expensive. Also it was a lot of fun to build it.
        return fimg

    def _find_foreground_mask(self):
        # Let's build a simple mask to separate pieces from background
        # just by checking if pixel is in range of some colours
        m = 0
        res = None

        # Here we calculate mask to separate background of the scanner
        scanner_bg = self._detect_scanner_background(self.orig_img)

        # And here we are trying to check different ranges for different
        # background to find the winner.
        hsv = cv2.cvtColor(self.orig_img, cv2.COLOR_BGR2HSV)

        for bg in self._backgrounds:
            mask = np.zeros(self.orig_img.shape[:2], np.uint8)

            for rng in bg:
                # As each pre-defined background is described by a list of
                # color ranges we are summing up their masks
                mask = cv2.bitwise_or(mask, cv2.inRange(hsv, rng[0], rng[1]))

            hist = cv2.calcHist([mask], [0], None, [2], [0, 256])
            # And here we are searching for a biggest possible mask across all
            # possible predefined backgrounds
            if hist[1] > m:
                m = hist[1]
                res = mask

        # then we remove scanner background
        res = cv2.bitwise_or(scanner_bg, res)
        res = cv2.morphologyEx(res, cv2.MORPH_OPEN, np.ones((3, 7), np.uint8))

        res = cv2.bitwise_not(res)
        # Init kernel for dilate/erode
        kernel = np.ones((5, 5), np.uint8)
        # optional blur of mask
        # mask = cv2.medianBlur(mask, 5)

        # Clean noise on background
        res = cv2.morphologyEx(res, cv2.MORPH_OPEN, kernel)
        # Clean noise inside pieces
        res = cv2.morphologyEx(res, cv2.MORPH_CLOSE, kernel)

        # Write original image with no background for debug purposes
        #cv2.imwrite("debug/mask.tif", mask)
        return res

    @property
    def _foreground_mask(self):
        if self._fg_mask is None:
            self._fg_mask = self._find_foreground_mask()
        return self._fg_mask

    def _make_shred(self, c, name, feature_extractors, sheet_name):
        """Creates a Shred instances from a given contour.

        Args:
            c: cv2 contour object.
            name: string shred name within a sheet.
            feature_extractors: iterable of AbstractShredFeature instances.

        Returns:
            A new Shred instance or None on failure.
        """
        height, width, channels = self.orig_img.shape

        # bounding rect of currrent contour
        r_x, r_y, r_w, r_h = cv2.boundingRect(c)

        # Generating simplified contour to use it in html
        epsilon = 0.01 * cv2.arcLength(c, True)
        simplified_contour = cv2.approxPolyDP(c, epsilon, True)

        # filter out too small fragments
        if self.px_to_mm(r_w) <= 3 or self.px_to_mm(r_h) <= 3:
            print("Skipping piece #%s as too small (%spx x %s px)" % (
                name, r_w, r_h))
            return None

        if self.px_to_mm(r_w) >= 100 and self.px_to_mm(r_h) >= 100:
            print("Skipping piece #%s as too big (%spx x %s px)" % (
                name, r_w, r_h))
            return None

        # position of rect of min area.
        # this will provide us angle to straighten image
        box_center, bbox, angle = cv2.minAreaRect(c)

        # We want our pieces to be "vertical"
        if bbox[0] > bbox[1]:
            angle += 90
            bbox = (bbox[1], bbox[0])

        if bbox[1] / float(bbox[0]) > 70:
            print("Skipping piece #%s as too too long and narrow" % name)
            return None

        # Coords of region of interest using which we should crop piece after
        # rotation
        y1 = math.floor(box_center[1] - bbox[1] / 2)
        x1 = math.floor(box_center[0] - bbox[0] / 2)
        bbox = tuple(map(int, map(math.ceil, bbox)))

        # A mask we use to show only piece we are currently working on
        piece_mask = np.zeros([height, width, 1], dtype=np.uint8)
        cv2.drawContours(piece_mask, [c], -1, 255, cv2.cv.CV_FILLED)

        # apply mask to original image
        img_crp = self.orig_img[r_y:r_y + r_h, r_x:r_x + r_w]
        piece_in_context = self.save_image(
            "pieces/%s_ctx" % name,
            self.orig_img[max(r_y - 10, 0):r_y + r_h + 10,
                          max(r_x - 10, 0):r_x + r_w + 10])

        mask = piece_mask[r_y:r_y + r_h, r_x:r_x + r_w]
        img_roi = cv2.bitwise_and(img_crp, img_crp, mask=mask)

        # Add alpha layer and set it to the mask
        img_roi = cv2.cvtColor(img_roi, cv2.cv.CV_BGR2BGRA)
        img_roi[:, :, 3] = mask[:, :, 0]

        # Straighten it
        # Because we crop original image before rotation we save us some memory
        # and a lot of time but we need to adjust coords of the center of
        # new min area rect
        M = cv2.getRotationMatrix2D((box_center[0] - r_x,
                                     box_center[1] - r_y), angle, 1)

        # And translate an image a bit to make it fit to the bbox again.
        # This is done with direct editing of the transform matrix.
        # (Wooohoo, I know matrix-fu)
        M[0][2] += r_x - x1
        M[1][2] += r_y - y1

        # Apply rotation/transform/crop
        img_roi = cv2.warpAffine(img_roi, M, bbox)
        piece_fname = self.save_image("pieces/%s" % name, img_roi, "png")

        # FEATURES MAGIC BELOW
        #
        # Get our mask/contour back after the trasnform
        _, _, _, mask = cv2.split(img_roi)

        contours, _ = cv2.findContours(mask.copy(), cv2.RETR_TREE,
                                       cv2.CHAIN_APPROX_SIMPLE)

        if len(contours) != 1:
            print("Piece #%s has strange contours after transform" % name)

        cnt = contours[0]

        features_fname = self.save_image("pieces/%s_mask" % name, mask, "png")

        base_features = {
            "angle": angle,
            "pos_x": r_x,
            "pos_y": r_y,
            "pos_width": r_w,
            "pos_height": r_h
        }

        tags_suggestions = []
        for feat in feature_extractors:
            fts, tags = feat.get_info(img_roi, cnt, name)
            base_features.update(fts)
            tags_suggestions += tags

        if tags_suggestions:
            print(name, tags_suggestions)

        return Shred(
            contour=c,
            features=base_features,
            features_fname=features_fname,
            img_roi=img_roi,
            name=name,
            piece_fname=piece_fname,
            piece_in_context_fname=piece_in_context,
            sheet=sheet_name,
            simplified_contour=simplified_contour,
            tags_suggestions=tags_suggestions,
        )



class AbstractShredFeature(object):
    def __init__(self, sheet):
        self.sheet = sheet

    def get_info(self, shred, contour, name):
        """Processes each individual shred and returns features and tags
        suggestions.

        Check colours.py and geometry.py for examples

        Args:
            shred: openCV image in RGBA mode, rotated and with no background.
            contour: openCV contour (list of coords).
            name: identifier of this shred. You might use it to save debug
            images

        Returns:
            dict with determined features
            tuple or list of tags suggestions (might be empty)

            For example:
            {
                "histogram": [...],
                "width": ...,
                "height": ...
            },
            (
                "has blue ink",
                "part of the picture"
            )
        """
        return {}, ()



import cv2
import itertools
from PIL import Image

from .base import AbstractShredFeature


def hex_to_bgr(hex_digits):
    """
    Convert a hexadecimal color value to a 3-tuple of integers
    """
    return tuple(int(s, 16) for s in (
        hex_digits[:2], hex_digits[2:4], hex_digits[4:]))


class ColourFeatures(AbstractShredFeature):
    palette_colours = [
        "000000",
        "434343",
        "666666",
        "999999",
        "b7b7b7",
        "cccccc",
        "d9d9d9",
        "efefef",
        "f3f3f3",
        "ffffff",
        "96000a",
        "fc0019",
        "fc992b",
        "fcff42",
        "06ff3b",
        "2cfffe",
        "5284e4",
        "2a00f9",
        "a4170e",
        "ca0012",
        "efc347",
        "69a956",
        "48818d",
        "4676d4",
        "4484c3",
        "694ba4",
        "a54b78",
        "831e13",
        "97000a",
        "b15f1b",
        "bd9024",
        "387727",
        "184f5b",
        "2652c8",
        "1a5290",
        "371872",
        "731746",
        "5a0d04",
        "650004",
        "773f10",
        "7e6015",
        "274f19",
        "0f343b",
        "234484",
        "103661",
        "22104b",
        "4b0f2f",
    ]

    skip = 10  # Amount of colours from the beginning of the list to ignore
               # white, black and shades of gray.

    def __init__(self, *args, **kwargs):
        super(ColourFeatures, self).__init__(*args, **kwargs)

        # Building our special palette.
        # Pillow is strange when it goes about palettes.
        # Here we are creating a new image and add some colours to it's
        # palette and rest of unused 256 is set to white. Otherwise
        # PIL will use shades of gray to fill unused ones.
        self.palette = Image.new("P", (1, 1), 0)

        # convert colours to the form suitable for set_palette
        plt = list(itertools.chain.from_iterable(
                   map(hex_to_bgr, self.palette_colours)))

        # padding the rest of palette with white colour
        self.palette.putpalette(
            plt + [255, 255, 255] * (256 - len(self.palette_colours)))

    def has_blue_ink(self, histogram):
        h = dict(zip(self.palette_colours, histogram))
        ratio = (h["694ba4"] + h["5284e4"] +
                 h["2a00f9"]) / float(sum(histogram))

        # Too much of blue inc is also bad (usually means a picture)
        if (ratio >= 0.01 and ratio <= 0.1 and
                not self.part_of_a_picture(histogram)):
            return True

    def part_of_a_picture(self, histogram):
        if sum(histogram[self.skip:]) / float(sum(histogram)) > 0.2:
            return True

    def has_yellow_marks(self, histogram):
        h = dict(zip(self.palette_colours, histogram))
        ratio = (h["efc347"] + h["fcff42"]) / float(sum(histogram))

        # Too much of blue inc is also bad (usually means a picture)
        if (ratio >= 0.05):
            return True

    # List of heuristics for particular features.
    tag_detectors = {
        "Has blue ink": has_blue_ink,
        "Part of a picture": part_of_a_picture,
        "Has yellow marks": has_yellow_marks,
    }

    def get_info(self, shred, contour, name):
        shred_pil = Image.fromarray(cv2.cvtColor(shred, cv2.COLOR_BGR2RGB))

        # downsampling image to our palette
        converted = shred_pil.quantize(palette=self.palette)
        # converted.save("../debug/converted%s.png" % name)

        mask = Image.fromarray(shred[:, :, 3])
        # Converting histogram with mask applied
        hist = converted.histogram(mask=mask)[:len(self.palette_colours)]

        total_pixels = sum(hist)
        hist_to_store = [0] * self.skip
        dominant_colours = []

        # Preparing clean version of histogram where only colours that has
        # > 1% is included. Same goes for the list of dominant colours.
        # Of course we are skipping white/black and shades of gray.
        for i, x in enumerate(self.palette_colours[self.skip:]):
            if hist[i + self.skip] > 0.01 * total_pixels:
                hist_to_store.append(hist[i + self.skip])
                dominant_colours.append(x)
            else:
                hist_to_store.append(0)

        tags = []
        for tag_name, tag_detector in self.tag_detectors.iteritems():
            if tag_detector(self, hist):
                tags.append(tag_name)

        return {
            "histogram_clean": hist_to_store,
            "histogram_full": hist,
            "colour_names": self.palette_colours,
            "dominant_colours": dominant_colours
        }, tuple(tags)



from argparse import ArgumentParser
from glob import glob
import os
import os.path
import shutil

import cv2
import exifread
from jinja2 import FileSystemLoader, Environment
import numpy as np

from features import GeometryFeatures, ColourFeatures
from sheet import Sheet


parser = ArgumentParser()
parser.add_argument(
    'fnames', type=str, help='Input files glob.', nargs='?',
    default="../src/puzzle_small.tif")
parser.add_argument(
    'out_format', type=str, help='Output format.', nargs='?',
    default="png")


def convert_poly_to_string(poly):
    # Helper to convert openCV contour to a string with coordinates
    # that is recognizible by html area tag
    return ",".join(map(lambda x: ",".join(map(str, x[0])), poly))

env = Environment(loader=FileSystemLoader("templates"))
env.filters["convert_poly_to_string"] = convert_poly_to_string


class SheetIO(object):
    """Encapsulates IO operations on sheets and shreds."""

    def __init__(self, fname, sheet_name, feature_extractors,
                 out_dir="out", out_format="png"):

        self.fname = fname
        self.sheet_name = sheet_name
        self.out_format = out_format
        self.out_dir = out_dir

        orig_img = cv2.imread(fname)
        self.sheet = Sheet(orig_img, dpi=self._dpi_from_exif(),
                           save_image=self.save_image)
        self.feature_extractors = [
            feat(self.sheet) for feat in feature_extractors]

    def _dpi_from_exif(self):
        def parse_resolution(val):
            x = map(int, map(str.strip, str(val).split("/")))
            if len(x) == 1:
                return x[0]
            elif len(x) == 2:
                return int(round(float(x[0]) / x[1]))
            else:
                raise ValueError

        with open(self.fname, "rb") as f:
            tags = exifread.process_file(f)

        if "Image XResolution" in tags and "Image YResolution" in tags:
            try:
                return (parse_resolution(tags["Image XResolution"]),
                        parse_resolution(tags["Image YResolution"]))
            except ValueError:
                return None
        return None

    def save_image(self, fname, img, format=None):
        full_out_dir, fname = os.path.split(
            os.path.join(self.out_dir, self.sheet_name, fname))

        try:
            os.makedirs(full_out_dir)
        except OSError:
            pass

        if format is None:
            format = self.out_format

        fname = "%s/%s.%s" % (full_out_dir, fname, format)
        cv2.imwrite(fname, img)

        return fname

    def overlay_contours(self):
        # Draw contours on top of image with a nice yellow tint
        overlay = np.zeros(self.sheet.orig_img.shape, np.uint8)
        contours = map(lambda x: x.contour, self.get_shreds())

        # Filled yellow poly.
        cv2.fillPoly(overlay, contours, [104, 255, 255])
        img = self.sheet.orig_img.copy() + overlay

        # Add green contour.
        cv2.drawContours(img, contours, -1, [0, 180, 0], 2)
        return img

    def export_results_as_html(self):
        img_with_overlay = self.overlay_contours()
        path_to_image = self.save_image("full_overlay", img_with_overlay)

        # Export one processes page as html for further review
        tpl = env.get_template("page.html")

        shreds = self.get_shreds()
        for c in shreds:
            # Slight pre-processing of the features of each piece
            c.features["angle"] = "%8.1f&deg;" % c.features["angle"]
            c.features["ratio"] = "%8.2f" % c.features["ratio"]
            c.features["solidity"] = "%8.2f" % c.features["solidity"]

        export_dir, img_name = os.path.split(path_to_image)

        with open("%s/index.html" % export_dir, "w") as fp:
            fp.write(tpl.render(
                img_name=img_name,
                contours=shreds,
                export_dir=export_dir,
                out_dir_name=self.sheet_name,
            ))

    def save_thumb(self, width=200):
        r = float(width) / self.sheet.orig_img.shape[1]
        dim = (width, int(self.sheet.orig_img.shape[0] * r))

        resized = cv2.resize(self.sheet.orig_img, dim,
                             interpolation=cv2.INTER_AREA)
        return self.save_image("thumb", resized)

    def get_shreds(self):
        return self.sheet.get_shreds(self.feature_extractors, self.sheet_name)


if __name__ == '__main__':
    args = parser.parse_args()

    fnames = args.fnames
    out_format = args.out_format

    out_dir = "../out"

    static_dir = os.path.join(out_dir, "static")
    if os.path.exists(static_dir):
        shutil.rmtree(static_dir)
    shutil.copytree("static", static_dir)

    sheets = []
    # Here we are processing all files one by one and also generating
    # index sheet for it
    for fname in glob(fnames):
        sheet_name = os.path.splitext(os.path.basename(fname))[0]

        print("Processing file %s" % fname)
        sheet = SheetIO(fname, sheet_name, [GeometryFeatures, ColourFeatures],
                        out_dir, out_format)

        sheet.export_results_as_html()
        sheets.append({
            "thumb": sheet.save_thumb(),
            "name": sheet.sheet_name
        })

    with open("../out/index.html", "w") as fp:
        tpl = env.get_template("index_sheet.html")

        fp.write(tpl.render(
            sheets=sheets
        ))



import cv2

from .base import AbstractShredFeature


class GeometryFeatures(AbstractShredFeature):
    def get_info(self, shred, contour, name):
        area = cv2.contourArea(contour)

        hull = cv2.convexHull(contour)
        hull_area = cv2.contourArea(hull)

        solidity = float(area) / hull_area

        # Also top and bottom points on the contour
        topmost = map(int, contour[contour[:, :, 1].argmin()][0])
        bottommost = map(int, contour[contour[:, :, 1].argmax()][0])
        _, _, r_w, r_h = cv2.boundingRect(contour)
        tags = []

        if solidity < 0.75:
            tags.append("Suspicious shape")

        width_mm = self.sheet.px_to_mm(r_w)
        height_mm = self.sheet.px_to_mm(r_h)
        ratio = shred.shape[0] / float(shred.shape[1])

        if width_mm > 6:
            tags.append("Suspicious width")

        return {
            "area": area,
            "ratio": ratio,
            "solidity": solidity,
            "topmost": topmost,
            "bottommost": bottommost,
            "width_mm": width_mm,
            "height_mm": height_mm
        }, tags


# # Let's also detect 25 of most outstanding corners of the contour.
# corners = cv2.goodFeaturesToTrack(mask, 25, 0.01, 10)
# corners = np.int0(corners) if corners is not None else []

# # edges = cv2.Canny(img_roi[:, :, 2], 100, 200)
# # save_image("pieces/%s_edges" % name, edges)

# # Draw contours for debug purposes
# mask = cv2.cvtColor(mask, cv2.cv.CV_GRAY2BGRA)
# mask[:, :, 3] = mask[:, :, 0]
# cv2.drawContours(mask, contours, -1, (255, 0, 255, 255), 2)

# # And put corners on top
# for i in corners:
#     x, y = i.ravel()
#     cv2.circle(mask, (x, y), 3, (0, 255, 0, 255), -1)

# cnt = contours[0]

# # Let's find features that will help us to determine top side of the piece
# # if mask.shape[0] / float(mask.shape[1]) >= 2.7:
# hull = cv2.convexHull(cnt, returnPoints=False)
# defects = cv2.convexityDefects(cnt, hull)

# # Check for convex defects to see if we can find a trace of a
# # shredder cut which is usually on top of the piece.

# if defects is not None:
#     has_defects = False
#     for i in range(defects.shape[0]):
#         s, e, f, d = defects[i, 0]
#         far = tuple(cnt[f][0])

#         # if convex defect is big enough
#         if d / 256. > 50:
#             has_defects = True
#             cv2.circle(mask, far, 5, [0, 0, 255, 255], -1)

#             # # And lays at the top or bottom of the piece
#             # y_dist = min(abs(0 - far[1]), abs(mask.shape[0] - far[1]))
#             # if float(y_dist) / mask.shape[0] < 0.1:
#             #     # and more or less is in the center
#             #     if abs(far[0] - mask.shape[1] / 2.) / mask.shape[1] < 0.25:
#             #         cv2.circle(mask, far, 5, [0, 0, 255, 255], -1)

#     # if has_defects:
#     #     cv2.imwrite("debug/def_20_%s_%s.png" % (self.sheet_name, name), mask)


from setuptools import setup
import os
from unshred import __version__

with open(os.path.join(os.path.dirname(__file__),
                       "requirements.txt")) as req_file:
    requirements = req_file.read().splitlines()

setup(
    name='unshred',
    version=__version__,
    url='https://github.com/dchaplinsky/unshred',
    license='MIT',
    author='Dmitry Chaplinsky',
    author_email='chaplinsky.dmitry@gmail.com',
    packages=["unshred",
              "unshred.features"],
    package_data={"unshred": ["static/*", "templates/*"]},
    description='Set of tools to analyze scanned shreds and tag them',
    platforms='any',
    install_requires=requirements,
    scripts=['unshred/split.py'],
)
